import bpy

from all_axis_render_camera.constants import CAMERA_TARGET_NAME
from all_axis_render_camera.utils import get_cam_setup


class CreateInvisibleTargetOperator(bpy.types.Operator):
    """
    Creates an object that's invisible to the camera on render.
    Can be used to set the center of the camera setup.
    """

    bl_idname = "object.create_invisible_target"
    bl_label = "Create Invisible Target"

    def execute(self, context):
        # Check if the object with the name already exists
        if CAMERA_TARGET_NAME in bpy.data.objects:
            obj = bpy.data.objects[CAMERA_TARGET_NAME]
            for collection in obj.users_collection:
                collection.objects.unlink(obj)
            # Remove the object from bpy.data.objects
            bpy.data.objects.remove(obj)

        # Create a Suzanne (the monkey) at the world origin
        bpy.ops.mesh.primitive_monkey_add(location=(0, 0, 0))
        suzanne = context.active_object

        if suzanne is None:
            self.report({"ERROR"}, "Failed to create the target object.")
            return {"CANCELLED"}

        suzanne.name = CAMERA_TARGET_NAME

        # Make Suzanne invisible to the camera on render
        suzanne.hide_render = True

        # Automatically select the center object as this Suzanne
        get_cam_setup(context).center_object = suzanne

        return {"FINISHED"}
