import os
import bpy


class RenderOperator(bpy.types.Operator):
    """Render cameras with default settings"""

    bl_idname = "scene.render_operator"
    bl_label = "Render"

    def execute(self, context):
        if "all_axis_cam" not in bpy.data.objects:
            self.report({"ERROR"}, "Camera not found")
            return {"CANCELLED"}

        output_directory = context.scene.render.filepath
        if not output_directory:
            self.report({"ERROR"}, "Output directory not set")
            return {"CANCELLED"}

        cam_object = bpy.data.objects["all_axis_cam"]
        context.scene.camera = cam_object

        context.scene.render.image_settings.file_format = "JPEG"
        context.scene.render.resolution_x = 2000
        context.scene.render.resolution_y = 2000

        # Enable transparency under Film settings
        context.scene.render.film_transparent = True

        # Update compositor settings
        scene = context.scene
        scene.use_nodes = True
        tree = scene.node_tree
        tree.nodes.clear()

        # Create render layers node
        render_layers = tree.nodes.new(type="CompositorNodeRLayers")

        # Create file output node for image
        file_output_image = tree.nodes.new(type="CompositorNodeOutputFile")
        file_output_image.base_path = output_directory
        file_output_image.file_slots[0].path = "####"

        # Create file output node for alpha
        file_output_alpha = tree.nodes.new(type="CompositorNodeOutputFile")
        file_output_alpha.base_path = os.path.join(output_directory, "alphas")
        file_output_alpha.file_slots[0].path = "####"

        # Create viewer node
        viewer_node = tree.nodes.new(type="CompositorNodeViewer")

        # Link nodes
        tree.links.new(render_layers.outputs["Image"], file_output_image.inputs[0])
        tree.links.new(render_layers.outputs["Alpha"], file_output_alpha.inputs[0])
        tree.links.new(render_layers.outputs["Image"], viewer_node.inputs[0])
        tree.links.new(render_layers.outputs["Alpha"], viewer_node.inputs[1])

        bpy.ops.render.render("INVOKE_DEFAULT", animation=True)

        return {"FINISHED"}
