import json
import bpy
import os
import numpy as np

from all_axis_render_camera.utils import get_cam_setup


def write_rendering_context(
    context: bpy.types.Context, camera: bpy.types.Object, filepath: str
):
    camera_setup_context = get_cam_setup(context)
    center_type = camera_setup_context.center_type
    center_object = camera_setup_context.center_object

    transform_matrix = np.eye(4)

    if center_type == "cursor":
        transform_matrix[:3, 3] = context.scene.cursor.location
    elif center_type == "object" and center_object:
        transform_matrix[:3, 3] = center_object.location
        rotation_matrix = center_object.matrix_world.to_3x3().normalized()
        transform_matrix[:3, :3] = rotation_matrix
    elif center_type == "origin":
        pass
    else:
        raise ValueError("Invalid center type")

    data = {
        "cam_rows": camera_setup_context.cam_rows,
        "cam_columns": camera_setup_context.cam_columns,
        "radius": camera_setup_context.radius,  # floating point number
        "structure_type": camera_setup_context.structure_type,  # "sphere" or "dome"
        "camera_sensor_width": camera.data.sensor_width,
        "camera_focal_length": camera.data.lens,
        "transform_matrix": transform_matrix.tolist(),
    }

    with open(filepath, "w") as f:
        json.dump(data, f, indent=2)


class SaveRenderingContextOperator(bpy.types.Operator):
    """Save camera positions to a file"""

    bl_idname = "scene.save_rendering_context"
    bl_label = "Save Rendering Context"

    @classmethod
    def poll(cls, context):
        # Disable the operator if the dataset directory is not set or "all_axis_cam" is not present
        return (
            bool(context.scene.render.filepath) and "all_axis_cam" in bpy.data.objects
        )

    def execute(self, context):
        write_rendering_context(
            context,
            bpy.data.objects["all_axis_cam"],
            os.path.join(context.scene.render.filepath, "rendering_settings.json"),
        )
        return {"FINISHED"}
