import bpy
from bpy.props import (
    IntProperty,
    FloatProperty,
    EnumProperty,
    PointerProperty,
    BoolProperty,
)

from .setup_cameras import update_camera_setup


def update_handler(self, context):
    update_camera_setup(context.scene)


class SetupCamsAddonProperties(bpy.types.PropertyGroup):
    cam_rows: IntProperty(
        name="Number of Rows",
        description="Number of rows in the camera setup",
        default=19,
        min=2,
        max=200,
        update=update_handler,
    )
    cam_columns: IntProperty(
        name="Number of Columns",
        description="Number of columns in the camera setup",
        default=36,
        min=2,
        max=200,
        update=update_handler,
    )
    radius: FloatProperty(
        name="Radius",
        description="Radius of the camera setup",
        default=3,
        min=0,
        max=100,
        update=update_handler,
    )
    structure_type: EnumProperty(
        name="Structure Type",
        description="Type of the camera setup",
        items=[("dome", "Dome", "Dome"), ("sphere", "Sphere", "Sphere")],
        default="sphere",
        update=update_handler,
    )
    center_type: EnumProperty(
        name="Center Type",
        description="Type of the center for the camera setup",
        items=[
            ("origin", "Origin", "Origin"),
            ("cursor", "Cursor", "Cursor"),
            ("object", "Object", "Object"),
        ],
        default="origin",
        update=update_handler,
    )
    center_object: PointerProperty(
        name="Center Object",
        description="Object to be the center of the camera setup",
        type=bpy.types.Object,
        update=update_handler,
    )
    camera_setup_follows_object: BoolProperty(
        name="Camera Setup Follows Object",
        description="If true, the camera setup will follow the object",
        default=True,
    )
